import { BaseCustomWebComponentConstructorAppend, css, html } from "@node-projects/base-custom-webcomponent";
import Chart from 'chart.js/auto/auto.js';
function requestAnimationFramePromise() {
    return new Promise(resolve => requestAnimationFrame(resolve));
}
export class ChartJsWebcomponent extends BaseCustomWebComponentConstructorAppend {
    static style = css `
            :host {
                display: block;
                position: relative;
            }
            
            canvas {
                width: 100%;
                height: 100%;
                position: absolute;
            }`;
    static template = html `<canvas id="root"></canvas>`;
    static is = 'node-projects-chart-js';
    static properties = {
        data: Object,
        borderColor: String,
        backgroundColor: String,
        enableXScale: Boolean,
        enableYScale: Boolean
    };
    #data;
    get data() {
        return this.#data;
    }
    set data(value) {
        this.#data = value;
        if (this.#ready) {
            this.#renderChart();
        }
    }
    #borderColor;
    get borderColor() {
        return this.#borderColor;
    }
    set borderColor(value) {
        this.#borderColor = value;
        if (this.#ready) {
            this.#renderChart();
        }
    }
    #backgroundColor;
    get backgroundColor() {
        return this.#backgroundColor;
    }
    set backgroundColor(value) {
        this.#backgroundColor = value;
        if (this.#ready) {
            this.#renderChart();
        }
    }
    #enableXScale = false;
    get enableXScale() {
        return this.#enableXScale;
    }
    set enableXScale(value) {
        this.#enableXScale = value;
        if (this.#ready) {
            this.#renderChart();
        }
    }
    #enableYScale = false;
    get enableYScale() {
        return this.#enableYScale;
    }
    set enableYScale(value) {
        this.#enableYScale = value;
        if (this.#ready) {
            this.#renderChart();
        }
    }
    #type;
    get type() {
        return this.#type;
    }
    set type(value) {
        if (this.#type != value) {
            if (this.#chart) {
                this.#chart.destroy();
                this.#chart = null;
            }
            this.#type = value;
            if (this.#ready) {
                this.#renderChart();
            }
        }
    }
    #root;
    #ready;
    #chart;
    constructor() {
        super();
        this._restoreCachedInititalValues();
        this.#root = this._getDomElement('root');
    }
    ready() {
        this._parseAttributesToProperties(true);
        this.#renderChart();
        this.#ready = true;
    }
    async #renderChart() {
        let labels;
        let values;
        if (Array.isArray(this.#data[0])) {
            labels = this.#data.map(row => row[1]);
            values = this.#data.map(row => row[0]);
        }
        else {
            labels = this.#data.map(row => row.label);
            values = this.#data.map(row => row.value);
        }
        //if (this.#chart)
        //    this.#chart.destroy();
        if (this.offsetWidth == 0 || this.offsetHeight == 0)
            await requestAnimationFramePromise();
        if (this.#chart) {
            this.#chart.data.labels = labels;
            this.#chart.data.datasets[0].data = values;
            this.#chart.data.datasets[0].borderColor = this.#borderColor;
            this.#chart.data.datasets[0].backgroundColor = this.#backgroundColor;
            this.#chart.options.scales.x.display = this.#enableXScale;
            this.#chart.options.scales.y.display = this.#enableYScale;
        }
        else {
            this.#chart = new Chart(this.#root, {
                type: this.#type,
                data: {
                    labels: labels,
                    datasets: [
                        {
                            data: values,
                            fill: true,
                            borderColor: this.#borderColor,
                            backgroundColor: this.#backgroundColor
                        }
                    ]
                },
                options: {
                    responsive: true,
                    maintainAspectRatio: false,
                    plugins: {
                        legend: {
                            display: false
                        },
                        tooltip: {
                            enabled: false
                        }
                    },
                    elements: {
                        point: {
                            radius: 0
                        }
                    },
                    scales: {
                        x: {
                            display: this.#enableXScale
                        },
                        y: {
                            display: this.#enableYScale
                        }
                    },
                }
            });
        }
    }
}
customElements.define(ChartJsWebcomponent.is, ChartJsWebcomponent);
